==> prime-0.7.1.tar.gz: prime-0.7.1/lib/makedict/dictformat.rb <==
# dictformat-cannadic.rb: Converter module for cannadic format.
# $Id: $
#
# Copyright (C) 2003 Hiroyuki Komatsu <komatsu@taiyaki.org>
#     All rights reserved.
#     This is free software with ABSOLUTELY NO WARRANTY.
#
# You can redistribute it and/or modify it under the terms of 
# the GNU General Public License version 2.

class DictFormat
  attr_accessor :dict

  def initialize
#    @data = {}
#    @pos_mapping = {}
    @dict = [{}, {}]
    @pos_unknown = "θ"
  end

  def load_external_dict (filename)
    File::open(filename, "r").each {|line|
      if items = parse(line.chomp) then
	items.each {|pron, pos, literal, freq, *attr|
	  set_word(pron, pos, literal, freq, *attr)
	}
      end
    }
  end

  def load_existent_dict (filename)
    return unless File::exist?(filename)

    File::open(filename, "r").each {|line|
      (pron, pos, literal, *rest) = line.chomp.split(/\t/)
      pos = (pos == @pos_unknown) ? nil : pos
      set_word(pron, pos, literal, *rest)
    }
  end

  def save_dict(filename)
    File::open(filename, "w") {|io|
      sort_result(@dict[0].values).each {|data|
	io.puts(dict_format(*data))
      }
    }
  end

  private
  ## This method should be over-written.
  def parse (line)
    return nil
  end

  def sort_result (results)
    results.sort {|result1, result2|
      (pron1, pos1, literal1, freq1, *rest1) = result1
      (pron2, pos2, literal2, freq2, *rest2) = result2

      if freq1 != freq2 then
	r = (freq2 <=> freq1)
      elsif pron1.length != pron2.length then
	r = (pron1.length <=> pron2.length)
      elsif pos1 != pos2 then
	if pos1 == @pos_unknown then
	  r = 1
	elsif pos2 == @pos_unknown then
	  r = -1
	else
	  r = 0
	end
      else
	r = (literal1 <=> literal2)
      end
      r
    }
  end

  def dict_format (pron, pos, literal, freq, *attr)
    return [pron, (pos or @pos_unknown), literal, freq, attr].join("\t").strip
  end

  def set_word(pron1, pos1, literal1, freq1, *attr1)
    if pron1.empty? or literal1.empty? then
      return false
    end

    (pron2, pos2, literal2, freq2, *attr2) = get_word(pron1, pos1, literal1)
    pron = pron1
    literal = literal1
    if pos1 and pos2.nil? then 
      @dict[0].delete(label(pron, nil, literal))
    end
    pos = (pos1 or pos2)

    freq = (freq1.to_i or 0)
    attr = attr2
    if pos1 == pos2 or pos1.nil? or pos2.nil? then
      freq  = [freq, freq2].max
      attr += attr2
    end

    @dict[0][label(pron, pos, literal)] = [pron, pos, literal, freq, *attr]
    if @dict[1][label(pron, literal)].nil? then
      @dict[1][label(pron, literal)] = pos
    end
    return true
  end

  def get_word (pron, pos, literal)
    word = (@dict[0][label(pron, pos, literal)] or
	    @dict[0][label(pron, @dict[1][label(pron, literal)], literal)] or
	    [pron, pos, literal, 0])
    return word
  end

  def label(*data)
    return data.join("\t")
  end
end

==> prime-0.7.4.tar.gz: prime-0.7.4/lib/makedict/dictformat.rb <==
# dictformat-cannadic.rb: Converter module for cannadic format.
# $Id: dictformat.rb,v 1.1.2.2 2003/12/29 09:50:35 komatsu Exp $
#
# Copyright (C) 2003 Hiroyuki Komatsu <komatsu@taiyaki.org>
#     All rights reserved.
#     This is free software with ABSOLUTELY NO WARRANTY.
#
# You can redistribute it and/or modify it under the terms of 
# the GNU General Public License version 2.

class DictFormat
  attr_accessor :dict

  def initialize (is_interactive = true)
    @is_interactive = is_interactive
#    @data = {}
#    @pos_mapping = {}
    @dict = [{}, {}]
    @pos_unknown = "θ"
  end

  def load_external_dict (filename)
    label = @is_interactive ? "INPUT_FILE LODING" : nil
    File::open(filename, "r").each_with_pbar(label) {|line|
      if items = parse(line.chomp) then
	items.each {|pron, pos, literal, freq, *attr|
	  set_word(pron, pos, literal, freq, *attr)
	}
      end
    }
  end

  def load_existent_dict (filename)
    return unless File::exist?(filename)

    label = @is_interactive ? "OUTPUT_FILE LODING" : nil
    File::open(filename, "r").each_with_pbar(label) {|line|
      (pron, pos, literal, *rest) = line.chomp.split(/\t/)
      pos = (pos == @pos_unknown) ? nil : pos
      set_word(pron, pos, literal, *rest)
    }
  end

  def save_dict(filename)
    label = @is_interactive ? "OUTPUT_FILE SAVING" : nil
    File::open(filename, "w") {|io|
      sort_result(@dict[0].values).each_with_pbar(label) {|data|
	io.puts(dict_format(*data))
      }
    }
  end

  private
  ## This method should be over-written.
  def parse (line)
    line.chomp!
    return [line.split(/\t/)]
  end

  def sort_result (results)
    label = @is_interactive ? "WORDS SORTING" : nil
    results.sort_with_pbar(label) {|result1, result2|
      (pron1, pos1, literal1, freq1, *rest1) = result1
      (pron2, pos2, literal2, freq2, *rest2) = result2

      if freq1 != freq2 then
	r = (freq2 <=> freq1)
      elsif pron1.length != pron2.length then
	r = (pron1.length <=> pron2.length)
      elsif pos1 != pos2 then
	if pos1 == @pos_unknown then
	  r = 1
	elsif pos2 == @pos_unknown then
	  r = -1
	else
	  r = 0
	end
      else
	r = (literal1 <=> literal2)
      end
      r
    }
  end

  def dict_format (pron, pos, literal, freq, *attr)
    return [pron, (pos or @pos_unknown), literal, freq, attr].join("\t").strip
  end

  def set_word(pron1, pos1, literal1, freq1 = 0, *attr1)
    if pron1.empty? or literal1.empty? then
      return false
    end

    (pron2, pos2, literal2, freq2, *attr2) = get_word(pron1, pos1, literal1)
    pron = pron1
    literal = literal1
    if pos1 and pos2.nil? then 
      @dict[0].delete(label(pron, nil, literal))
    end
    pos = (pos1 or pos2)

    freq = (freq1.to_i or 0)
    attr = attr2
    if pos1 == pos2 or pos1.nil? or pos2.nil? then
      freq  = [freq, freq2].max
      attr += attr2
    end

    @dict[0][label(pron, pos, literal)] = [pron, pos, literal, freq, *attr]
    if @dict[1][label(pron, literal)].nil? then
      @dict[1][label(pron, literal)] = pos
    end
    return true
  end

  def get_word (pron, pos, literal)
    word = (@dict[0][label(pron, pos, literal)] or
	    @dict[0][label(pron, @dict[1][label(pron, literal)], literal)] or
	    [pron, pos, literal, 0])
    return word
  end

  def label(*data)
    return data.join("\t")
  end
end
